# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Page'
        db.create_table('ccpages_page', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('password', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('hash', self.gf('django.db.models.fields.CharField')(max_length=40, null=True, blank=True)),
            ('slug', self.gf('django.db.models.fields.SlugField')(unique=True, max_length=50)),
            ('content', self.gf('django.db.models.fields.TextField')()),
            ('content_rendered', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('parent', self.gf('mptt.fields.TreeForeignKey')(blank=True, related_name='children', null=True, to=orm['ccpages.Page'])),
            ('order', self.gf('django.db.models.fields.DecimalField')(default=10.0, max_digits=8, decimal_places=2)),
            ('status', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=1)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, null=True, blank=True)),
            ('modified', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, null=True, blank=True)),
            ('lft', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
            ('rght', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
            ('tree_id', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
            ('level', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
        ))
        db.send_create_signal('ccpages', ['Page'])

        # Adding model 'PageImage'
        db.create_table('ccpages_pageimage', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('page', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['ccpages.Page'])),
            ('src', self.gf('ccthumbs.fields.ImageWithThumbsField')(blank=False)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('order', self.gf('django.db.models.fields.DecimalField')(default=10.0, max_digits=8, decimal_places=2)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, null=True, blank=True)),
            ('modified', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, null=True, blank=True)),
        ))
        db.send_create_signal('ccpages', ['PageImage'])

        # Adding model 'PageAttachment'
        db.create_table('ccpages_pageattachment', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('page', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['ccpages.Page'])),
            ('src', self.gf('django.db.models.fields.files.FileField')(max_length=100)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('order', self.gf('django.db.models.fields.DecimalField')(default=10.0, max_digits=8, decimal_places=2)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, null=True, blank=True)),
            ('modified', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, null=True, blank=True)),
        ))
        db.send_create_signal('ccpages', ['PageAttachment'])


    def backwards(self, orm):
        # Deleting model 'Page'
        db.delete_table('ccpages_page')

        # Deleting model 'PageImage'
        db.delete_table('ccpages_pageimage')

        # Deleting model 'PageAttachment'
        db.delete_table('ccpages_pageattachment')


    models = {
        'ccpages.page': {
            'Meta': {'ordering': "['order']", 'object_name': 'Page'},
            'content': ('django.db.models.fields.TextField', [], {}),
            'content_rendered': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'hash': ('django.db.models.fields.CharField', [], {'max_length': '40', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'order': ('django.db.models.fields.DecimalField', [], {'default': '10.0', 'max_digits': '8', 'decimal_places': '2'}),
            'parent': ('mptt.fields.TreeForeignKey', [], {'blank': 'True', 'related_name': "'children'", 'null': 'True', 'to': "orm['ccpages.Page']"}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '50'}),
            'status': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '1'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'})
        },
        'ccpages.pageattachment': {
            'Meta': {'object_name': 'PageAttachment'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'order': ('django.db.models.fields.DecimalField', [], {'default': '10.0', 'max_digits': '8', 'decimal_places': '2'}),
            'page': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ccpages.Page']"}),
            'src': ('django.db.models.fields.files.FileField', [], {'max_length': '100'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'})
        },
        'ccpages.pageimage': {
            'Meta': {'object_name': 'PageImage'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'order': ('django.db.models.fields.DecimalField', [], {'default': '10.0', 'max_digits': '8', 'decimal_places': '2'}),
            'page': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ccpages.Page']"}),
            'src': ('ccthumbs.fields.ImageWithThumbsField', [], {'blank': 'False'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['ccpages']


import ccsitemaps
from ccpages.models import Page


class PageSiteMap(ccsitemaps.SiteMap):
    model = Page

    @staticmethod
    def last_mod():
        try:
            last_mod = Page.objects\
                .visible()\
                .order_by('-modified')[0]
            return last_mod.modified
        except IndexError:
            return None

    @staticmethod
    def get_objects():
        return Page.objects.visible()


ccsitemaps.register(PageSiteMap)



import hashlib
from django import forms
from mptt.forms import TreeNodeChoiceField
from mptt.admin import MPTTAdminForm
from writingfield import FullScreenTextarea
from ccpages.models import Page


class PageAdminForm(MPTTAdminForm):

    class Meta:
        model = Page
        widgets = {
            'content': FullScreenTextarea()
        }
    class Media:
        css = {
                'screen': ('ccpages/css/admin.css',)
            }

class PagePasswordForm(forms.Form):

    password = forms.CharField(widget=forms.PasswordInput())

    def __init__(self, *args, **kwargs):
        self.page = kwargs.pop('page', None)
        super(PagePasswordForm, self).__init__(*args, **kwargs)


    def clean_password(self):
        page_hash = self.page.hash
        password = self.cleaned_data['password']
        user_hash = hashlib.sha1(password).hexdigest()
        if user_hash == page_hash:
            self.hash = user_hash
            return password
        raise forms.ValidationError('Incorrect password')



from django import template
from django.conf import settings
from ccpages.models import Page

register = template.Library()

@register.inclusion_tag('ccpages/_js.html')
def ccpages_js():
    return {
        'STATIC_URL': settings.STATIC_URL,
    }

@register.inclusion_tag('ccpages/_css.html')
def ccpages_css():
    return {
        'STATIC_URL': settings.STATIC_URL,
    }

@register.inclusion_tag('ccpages/_nav_breadcrumb.html')
def ccpages_nav_breadcrumbs(page):
    """returns a breadcrumb"""
    return {
        'pages': Page.objects.nav_breadcrumbs(page),
        'page': page,
    }

@register.inclusion_tag('ccpages/_nav_local.html')
def ccpages_nav_local(page):
    """returns the local nav for a given page's root"""
    return {
        'pages': Page.objects.nav_local(page)
    }


@register.assignment_tag
def ccpages_nav_global():
    """returns the global pages"""
    return Page.objects.nav_global()




from django.db import models
from mptt.models import MPTTModel, TreeForeignKey
from ccthumbs.fields import ImageWithThumbsField
from ccpages import settings as c_settings
from ccpages import listeners
from ccpages.managers import PagesManager


class Page(MPTTModel):
    """ The page model is a simple model designed to be used
    as the basis for a rudimentary cms"""
    VISIBLE = 1
    HIDDEN = 0
    STATUS_CHOICES = (
            (HIDDEN, 'Hidden'),
            (VISIBLE, 'Visible')
    )
    title = models.CharField(
            max_length=255)
    password = models.CharField(
            max_length=255,
            blank=True,
            null=True)
    hash = models.CharField(
            max_length=40,
            blank=True,
            null=True)
    slug = models.SlugField(
            unique=True)
    content = models.TextField()
    content_rendered = models.TextField(
            blank=True,
            null=True)
    parent = TreeForeignKey('self',
            blank=True,
            null=True,
            related_name='children')
    order = models.DecimalField(
            default=10.00,
            max_digits=8,
            decimal_places=2)
    status = models.PositiveSmallIntegerField(
            default=c_settings.CCPAGES_DEFAULT_STATUS,
            choices=STATUS_CHOICES)
    layout = models.CharField(
            choices=c_settings.CCPAGES_LAYOUTS,
            default='ccpages/view.html',
            max_length=255)
    created = models.DateTimeField(
            blank=True,
            null=True,
            auto_now_add=True)
    modified = models.DateTimeField(
            blank=True,
            null=True,
            auto_now=True)

    custom = PagesManager()

    class Meta:
        ordering = ['order']

    def __unicode__(self):
        return u'%s' % self.title

    @models.permalink
    def get_absolute_url(self):
        return ('ccpages:view', (),{
            'slug': self.slug})


    @property
    def description(self):
        return self.content_rendered[:100]

class PageImage(models.Model):
    page = models.ForeignKey(Page)
    url = models.CharField(
            blank=True,
            null=True,
            max_length=255)
    src = ImageWithThumbsField(
            upload_to='ccpages/%Y/%m/%d',
            sizes=c_settings.CCPAGES_IMAGE_SIZES)
    title = models.CharField(
            max_length=255,
            null=True,
            blank=True)
    order = models.DecimalField(
            default=10.00,
            max_digits=8,
            decimal_places=2)
    created = models.DateTimeField(
            blank=True,
            null=True,
            auto_now_add=True)
    modified = models.DateTimeField(
            blank=True,
            null=True,
            auto_now=True)


    class Meta:
        ordering = ['order']


    def __unicode__(self):
        return u'page image: %s' % self.pk

class PageAttachment(models.Model):
    page = models.ForeignKey(Page)
    src = models.FileField(
            upload_to='ccpages/%Y/%m/%d')
    title = models.CharField(
            max_length=255,
            null=True,
            blank=True)
    order = models.DecimalField(
            default=10.00,
            max_digits=8,
            decimal_places=2)
    created = models.DateTimeField(
            blank=True,
            null=True,
            auto_now_add=True)
    modified = models.DateTimeField(
            blank=True,
            null=True,
            auto_now=True)

    def __unicode__(self):
        return u'page attachment: %s' % self.pk


models.signals.pre_save.connect(
        listeners.set_attachment_title,
        sender=PageAttachment,
        dispatch_uid='ccpages_set_attachment_title')

models.signals.pre_save.connect(
        listeners.set_content_rendered,
        sender=Page,
        dispatch_uid='page_set_content_rendered')

models.signals.pre_save.connect(
        listeners.set_hash,
        sender=Page,
        dispatch_uid='page_set_hash')



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'Page.layout'
        db.alter_column('ccpages_page', 'layout', self.gf('django.db.models.fields.FilePathField')(max_length=255, path='/Users/jcurle/Sites/istanbul/istanbul/istanbul//templates/ccpages/layouts/', null=True, match='[\\w\\.\\-]+\\.html'))

    def backwards(self, orm):

        # Changing field 'Page.layout'
        db.alter_column('ccpages_page', 'layout', self.gf('django.db.models.fields.FilePathField')(path='/Users/jcurle/Sites/istanbul/istanbul/istanbul//templates/ccpages/layouts/', max_length=100, null=True, match='[\\w\\.\\-]+\\.html'))

    models = {
        'ccpages.page': {
            'Meta': {'ordering': "['order']", 'object_name': 'Page'},
            'content': ('django.db.models.fields.TextField', [], {}),
            'content_rendered': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'hash': ('django.db.models.fields.CharField', [], {'max_length': '40', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'layout': ('django.db.models.fields.FilePathField', [], {'max_length': '255', 'path': "'/Users/jcurle/Sites/istanbul/istanbul/istanbul//templates/ccpages/layouts/'", 'null': 'True', 'match': "'[\\\\w\\\\.\\\\-]+\\\\.html'", 'blank': 'True'}),
            'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'order': ('django.db.models.fields.DecimalField', [], {'default': '10.0', 'max_digits': '8', 'decimal_places': '2'}),
            'parent': ('mptt.fields.TreeForeignKey', [], {'blank': 'True', 'related_name': "'children'", 'null': 'True', 'to': "orm['ccpages.Page']"}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '50'}),
            'status': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '1'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'})
        },
        'ccpages.pageattachment': {
            'Meta': {'object_name': 'PageAttachment'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'order': ('django.db.models.fields.DecimalField', [], {'default': '10.0', 'max_digits': '8', 'decimal_places': '2'}),
            'page': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ccpages.Page']"}),
            'src': ('django.db.models.fields.files.FileField', [], {'max_length': '100'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'})
        },
        'ccpages.pageimage': {
            'Meta': {'object_name': 'PageImage'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'order': ('django.db.models.fields.DecimalField', [], {'default': '10.0', 'max_digits': '8', 'decimal_places': '2'}),
            'page': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ccpages.Page']"}),
            'src': ('ccthumbs.fields.ImageWithThumbsField', [], {'blank': 'False'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'url': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['ccpages']


import os
from unittest import skipUnless
from decimal import Decimal
from django.test import TestCase
from django.conf import settings
from django.core.files import File
from ccpages.forms import PagePasswordForm
from ccpages.models import Page, PageAttachment

class ListenerTestCases(TestCase):

    @skipUnless(os.path.exists('%s/ccpages/test.pdf' % settings.STATIC_ROOT),
            'test.pdf file does not exist')
    def test_title(self):
        """A title is set on a file from filename is none is supplied"""
        # open file
        test_pdf = open('%s/ccpages/test.pdf' % settings.STATIC_ROOT)
        # make page and attachment
        p1 = Page()
        p1.title = '1'
        p1.slug = '1'
        p1.content = '# Hello World'
        p1.order = Decimal('1')
        p1.password = 'ha'
        p1.status = Page.VISIBLE
        p1.save()
        at1 = PageAttachment()
        at1.page = p1
        at1.src = File(test_pdf, 'ccpages/test.pdf')
        at1.save()
        # the title is 'test.pdf'
        self.assertEqual(at1.title, 'test.pdf')
        test_pdf.close()
        os.unlink(at1.src.path)
        # make another one, but this time with a title
        test_pdf = open('%s/ccpages/test.pdf' % settings.STATIC_ROOT)
        at2 = PageAttachment()
        at2.page = p1
        at2.src = File(test_pdf, 'ccpages/test.pdf')
        at2.title = 'Arther'
        at2.save()
        # title is now arther
        self.assertEqual(at2.title, 'Arther')
        # delete the files
        test_pdf.close()
        os.unlink(at2.src.path)

    def test_content_rendered(self):
        """When a page is saved the content is passed through
        markdown and saved as content_rendered"""
        page1 = Page()
        page1.title = '1'
        page1.slug = '1'
        page1.content = '# Hello World'
        page1.order = Decimal('1')
        page1.password = 'ha'
        page1.status = Page.VISIBLE
        page1.save()
        # we now have rendered content
        self.assertHTMLEqual(
                page1.content_rendered,
                '<h1 id="hello-world">\nHello World\n</h1>')

    def test_hash_if_password(self):
        """A hash is generated on save if page has password"""
        page1 = Page()
        page1.title = '1'
        page1.slug = '1'
        page1.content = '1'
        page1.order = Decimal('1')
        page1.password = 'ha'
        page1.status = Page.VISIBLE
        page1.save()
        # get the page
        p = Page.objects.get(pk=page1.pk)
        # we have a hash
        self.assertEqual(
                p.hash,
                'f9fc27b9374ad1e3bf34fdbcec3a4fd632427fed')
    
    def test_hash_if_no_password(self):
        """A hash is not generated on save if page has no password"""
        page1 = Page()
        page1.title = '1'
        page1.slug = '1'
        page1.content = '1'
        page1.order = Decimal('1')
        page1.status = Page.VISIBLE
        page1.save()
        # get the page
        p = Page.objects.get(pk=page1.pk)
        # we have no hash
        self.assertFalse(p.hash)



from django.conf import settings

"""The default status for pages. 1 is visible and 0 is hidden"""
CCPAGES_DEFAULT_STATUS = getattr(
            settings,
            'CCPAGES_DEFAULT_STATUS',
            1)

"""Sizes for the images in the PageImage model"""
CCPAGES_IMAGE_SIZES = getattr(
        settings,
        'CCPAGES_IMAGE_SIZES',
        (   (140, 140),
            (240,160),
            (480,320),
            (960,640)))

CCPAGES_LAYOUTS = (
    ('ccpages/view.html', 'default'),
)
try:
    CCPAGES_LAYOUTS += settings.CCPAGES_LAYOUTS
except AttributeError:
    pass



"""Path to the custom layouts"""
try:
    layouts = settings.TEMPLATE_DIRS[0] + 'ccpages/layouts/'
except IndexError:
    layouts = None
CUSTOM_LAYOUTS = getattr(
        settings,
        'CCPAGES_CUSTOM_LAYOUTS',
        layouts)



# -*- coding: utf-8 -*-
import os
from django.utils.encoding import smart_str
import hashlib
from markdown2 import markdown

def set_attachment_title(sender, instance, **kwargs):
    """If a file attachment is saved and it does not
    have a title then one will be generated from it's filename"""
    if not instance.title:
        instance.title = os.path.basename(instance.src.path)

def set_content_rendered(sender, instance, **kwargs):
    """When a page is saved the content is rendered out
    onto the model and saved to avoid the overhead of doing
    it on every request"""
    content_rendered = markdown(
                        instance.content,
                        extras=[
                            'header-ids',
                            'smarty-pants'])
    instance.content_rendered = smart_str(content_rendered)





def set_hash(sender, instance, **kwargs):
    """If a page has a password then create a hash for it"""

    if instance.password is None:
        return None

    instance.hash = hashlib.sha1(instance.password).hexdigest()



from haystack import indexes
from haystack import site
from ccpages.models import Page

class PageIndex(indexes.SearchIndex):
    text = indexes.CharField(
            document=True,
            use_template=True)
    title = indexes.CharField(
            model_attr='title')
    description = indexes.CharField(
            model_attr='description')
    created = indexes.DateTimeField(
            model_attr='created')

    def index_queryset(self):
        return Page.objects.filter(
                password='',
                status=Page.VISIBLE)

site.register(Page, PageIndex)
